@@ -3,7 +3,7 @@ package Inline::Java::Class ;
 use strict ;
 use Carp ;
 
-$Inline::Java::Class::VERSION = '0.47' ;
+$Inline::Java::Class::VERSION = '0.49' ;
 
 $Inline::Java::Class::MAX_SCORE = 10 ;
 
@@ -128,10 +128,10 @@ sub CastArgument {
 
 	my $sub = sub {
 		my $array_type = undef ;
-		if ((defined($arg))&&(UNIVERSAL::isa($arg, "Inline::Java::Class::Cast"))){
-			my $v = $arg->get_value() ;
-			$proto = $arg->get_type() ;
-			$array_type = $arg->get_array_type() ;
+		if ((defined($arg))&&(UNIVERSAL::isa($arg, "Inline::Java::Class::Coerce"))){
+			my $v = $arg->__get_value() ;
+			$proto = $arg->__get_type() ;
+			$array_type = $arg->__get_array_type() ;
 			$arg = $v ;
 		}
 
@@ -158,7 +158,10 @@ sub CastArgument {
 			else{
 				if (ref($arg)){
 					# We got some other type of ref...
-					croak "Can't convert $arg to object $proto" ;
+					if ($arg !~ /^(.*?)=/){
+						# We do not have a blessed reference, so ...
+						croak "Can't convert $arg to object $proto" ;
+					}
 				}
 				else{
 					# Here we got a scalar
@@ -246,26 +249,47 @@ sub CastArgument {
 			# Here the prototype calls for an object of type $proto
 			# We must ask Java if our object extends $proto		
 			if (ref($arg)){
-				my ($msg, $score) = $arg->__isa($proto) ;
-				if ($msg){
-					croak $msg ;
-				}
-				Inline::Java::debug(3, "$arg is a $proto") ;
-
-				# a matching object, pretty good match, except if proto
-				# is java.lang.Object
-				if ($proto eq "java.lang.Object"){	
-					return ($arg, 1) ;
-				}
+				if ((UNIVERSAL::isa($arg, "Inline::Java::Object"))||(UNIVERSAL::isa($arg, "Inline::Java::Array"))){
+					my ($msg, $score) = $arg->__isa($proto) ;
+					if ($msg){
+						croak $msg ;
+					}
+					Inline::Java::debug(3, "$arg is a $proto") ;
+	
+					# a matching object, pretty good match, except if proto
+					# is java.lang.Object
+					if ($proto eq "java.lang.Object"){	
+						return ($arg, 1) ;
+					}
 				
-				# Here we deduce points the more our argument is "far"
-				# from the prototype.
-				if (! UNIVERSAL::isa($arg, "Inline::Java::Array")){
-					return ($arg, 7 - ($score * 0.01)) ;
+					# Here we deduce points the more our argument is "far"
+					# from the prototype.
+					if (! UNIVERSAL::isa($arg, "Inline::Java::Array")){
+						return ($arg, 7 - ($score * 0.01)) ;
+					}
+					else{
+						# We need to keep the array score somewhere...
+						return ($arg, $array_score) ;
+					}
 				}
-				else{
-					# We need to keep the array score somewhere...
-					return ($arg, $array_score) ;
+				else {
+					# We want to send a Perl object to the Java side.
+					my $ijp = new Inline::Java::Protocol(undef, $inline) ;
+					my $score = $ijp->__ISA($proto, 'org.perl.inline.java.InlineJavaPerlObject') ;
+					if ($score == -1){
+						croak "$proto is not a kind of org.perl.inline.java.InlineJavaPerlObject" ;
+					}
+
+					Inline::Java::debug(3, "$arg is a $proto") ;
+					
+					# a matching object, pretty good match, except if proto
+					# is java.lang.Object
+					if ($proto eq "java.lang.Object"){	
+						return ($arg, 1) ;
+					}
+					else{
+						return ($arg, 7 - ($score * 0.01)) ;
+					}
 				}
 			}
 
@@ -277,17 +301,17 @@ sub CastArgument {
 
 	my @ret = $sub->() ;
 
-	if ((defined($arg_ori))&&(UNIVERSAL::isa($arg_ori, "Inline::Java::Class::Cast"))){
+	if ((defined($arg_ori))&&(UNIVERSAL::isa($arg_ori, "Inline::Java::Class::Coerce"))){
 		# It seems we had casted the variable to a specific type
-		if ($arg_ori->matches($proto_ori)){
-			Inline::Java::debug(3, "type cast match!") ;
+		if ($arg_ori->__matches($proto_ori)){
+			Inline::Java::debug(3, "type coerce match!") ;
 			$ret[1] = $Inline::Java::Class::MAX_SCORE ;
 		}
 		else{
-			# We have casted to something that doesn't exactly match
+			# We have coerced to something that doesn't exactly match
 			# any of the available types. 
 			# For now we don't allow this.
-			croak "Cast ($proto) doesn't exactly match prototype ($proto_ori)" ;
+			croak "Coerce ($proto) doesn't exactly match prototype ($proto_ori)" ;
 		}
 	}
 
@@ -424,8 +448,8 @@ sub ClassIsArray {
 
 
 
-######################## Inline::Java::Class::Cast ########################
-package Inline::Java::Class::Cast ;
+######################## Inline::Java::Class::Coerce ########################
+package Inline::Java::Class::Coerce ;
 
 
 use Carp ;
@@ -436,7 +460,7 @@ sub new {
 	my $value = shift ;
 	my $array_type = shift ;
 
-	if (UNIVERSAL::isa($value, "Inline::Java::Class::Cast")){
+	if (UNIVERSAL::isa($value, "Inline::Java::Class::Coerce")){
 		# This allows chaining
 		$value = $value->get_value() ;
 	}
@@ -451,27 +475,27 @@ sub new {
 }
 
 
-sub get_value {
+sub __get_value {
 	my $this = shift ;
 
 	return $this->{value} ;
 }
 
 
-sub get_type {
+sub __get_type {
 	my $this = shift ;
 
 	return $this->{cast} ;
 }
 
-sub get_array_type {
+sub __get_array_type {
 	my $this = shift ;
 
 	return $this->{array_type} ;
 }
 
 
-sub matches {
+sub __matches {
 	my $this = shift ;
 	my $proto = shift ;
 
