@@ -3,11 +3,12 @@ package Inline::Java::JVM ;
 
 use strict ;
 use Carp ;
-use IPC::Open3 ;
 use IO::File ;
+use IPC::Open3 ;
 use IO::Socket ;
+use Inline::Java::Portable ;
 
-$Inline::Java::JVM::VERSION = '0.47' ;
+$Inline::Java::JVM::VERSION = '0.49' ;
 
 my %SIGS = () ;
 
@@ -62,12 +63,13 @@ sub new {
 		my $debug = Inline::Java::get_DEBUG() ;
 
 		$this->{shared} = $o->get_java_config('SHARED_JVM') ;
+		$this->{start_jvm} = $o->get_java_config('START_JVM') ;
 		$this->{port} = $o->get_java_config('PORT') ;
 		$this->{host} = "localhost" ;
 
 		# Grab the next free port number and release it.
 		if ((! $this->{shared})&&($this->{port} < 0)){
-			if (Inline::Java::portable("GOT_NEXT_FREE_PORT")){
+			if (Inline::Java::Portable::portable("GOT_NEXT_FREE_PORT")){
 				my $sock = IO::Socket::INET->new(
 					Listen => 0, Proto => 'tcp',
 					LocalAddr => 'localhost', LocalPort => 0) ;
@@ -101,15 +103,19 @@ sub new {
 				Inline::Java::debug(1, "connected to already running JVM!") ;
 				return $this ;
 			}
+
+			if (! $this->{start_jvm}){
+				croak("Can't find running JVM and START_JVM = 0") ;
+			}
 		}
 
 		my $java = File::Spec->catfile($o->get_java_config('J2SDK'), 'bin',
 			($this->{debugger} ? "jdb" : "java") . 
-			Inline::Java::portable("EXE_EXTENSION")) ;
+			Inline::Java::Portable::portable("EXE_EXTENSION")) ;
 
 		my $shared = ($this->{shared} ? "true" : "false") ;
 		my $priv = ($this->{private} ? "true" : "false") ;
-		my $cmd = Inline::Java::portable("SUB_FIX_CMD_QUOTES", "\"$java\" $args org.perl.inline.java.InlineJavaServer $debug $this->{port} $shared $priv") ;
+		my $cmd = Inline::Java::Portable::portable("SUB_FIX_CMD_QUOTES", "\"$java\" $args org.perl.inline.java.InlineJavaServer $debug $this->{port} $shared $priv") ;
 		Inline::Java::debug(2, $cmd) ;
 		if ($o->get_config('UNTAINT')){
 			($cmd) = $cmd =~ /(.*)/ ;
@@ -117,7 +123,7 @@ sub new {
 
 		my $pid = 0 ;
 		eval {
-			$pid = $this->launch($cmd) ;
+			$pid = $this->launch($o, $cmd) ;
 		} ;
 		croak "Can't exec JVM: $@" if $@ ;
 
@@ -130,7 +136,7 @@ sub new {
 		}
 
 		$this->{pid} = $pid ;
-		$this->{socket}	= $this->setup_socket(
+		$this->{socket}	= setup_socket(
 			$this->{host}, 
 			$this->{port}, 
 			# Give the user an extra hour's time set breakpoints and the like...
@@ -145,15 +151,17 @@ sub new {
 
 sub launch {
 	my $this = shift ;
+	my $o = shift ;
 	my $cmd = shift ;
 
 	local $SIG{__WARN__} = sub {} ;
 
-	my $dn = Inline::Java::portable("DEV_NULL") ;
+	my $dn = Inline::Java::Portable::portable("DEV_NULL") ;
 	my $in = ($this->{debugger} ? ">&STDIN" : new IO::File("<$dn")) ;
 	if (! defined($in)){
 		croak "Can't open $dn for reading" ;
 	}
+
 	my $out = ">&STDOUT" ;
 	if ($this->{shared}){
 		$out = new IO::File(">$dn") ;
@@ -161,7 +169,10 @@ sub launch {
 			croak "Can't open $dn for writing" ;
 		}
 	}
-	my $pid = open3($in, $out, ">&STDERR", $cmd) ;
+
+	my $err = ">&STDERR" ;
+
+	my $pid = open3($in, $out, $err, $cmd) ;
 
 	if (! $this->{debugger}){
 		close($in) ;
@@ -240,8 +251,9 @@ sub shutdown {
 }
 
 
+# This cannot be a member function because it can be used
+# elsewhere to connect to the JVM.
 sub setup_socket {
-	my $this = shift ;
 	my $host = shift ;
 	my $port = shift ;
 	my $timeout = shift ;
@@ -250,7 +262,7 @@ sub setup_socket {
 	my $socket = undef ;
 
 	my $last_words = "timeout\n" ;
-	my $got_alarm = Inline::Java::portable("GOT_ALARM") ;
+	my $got_alarm = Inline::Java::Portable::portable("GOT_ALARM") ;
 
 	eval {
 		local $SIG{ALRM} = sub { die($last_words) ; } ;
@@ -315,7 +327,7 @@ sub reconnect {
 		$this->{socket} = undef ;
 	}
 
-	my $socket = $this->setup_socket(
+	my $socket = setup_socket(
 		$this->{host}, 
 		$this->{port}, 
 		0,
