@@ -1,7 +1,6 @@
 package Inline::Java::Portable ;
 @Inline::Java::Portable::ISA = qw(Exporter) ;
 
-@EXPORT = qw(portable make_classpath get_server_jar get_user_jar get_source_dir) ;
 
 use strict ;
 use Exporter ;
@@ -10,7 +9,7 @@ use Config ;
 use File::Find ;
 use File::Spec ;
 
-$Inline::Java::Portable::VERSION = '0.47' ;
+$Inline::Java::Portable::VERSION = '0.49' ;
 
 # Here is some code to figure out if we are running on command.com
 # shell under Windows.
@@ -46,7 +45,7 @@ sub make_classpath {
 	}
 	push @list, @paths ;
 
-	my $sep = portable("ENV_VAR_PATH_SEP_CP") ;
+	my $sep = Inline::Java::Portable::portable("ENV_VAR_PATH_SEP_CP") ;
 	my @cp = split(/$sep+/, join($sep, @list)) ;
 
 	# Clean up paths
@@ -62,7 +61,7 @@ sub make_classpath {
 		if (($p)&&(-e $p)){
 			if ($cp{$p}){
 				my $fp = (-d $p ? File::Spec->rel2abs($p) : $p) ;
-				push @fcp, portable("SUB_FIX_CLASSPATH", $fp) ;
+				push @fcp, Inline::Java::Portable::portable("SUB_FIX_CLASSPATH", $fp) ;
 				delete $cp{$p} ;
 			}
 		}
@@ -158,6 +157,9 @@ sub portable {
 		SUB_FIX_MAKE_QUOTES	=>	undef,
 		JVM_LIB				=>	"libjvm.$Config{dlext}",
 		JVM_SO				=>	"libjvm.$Config{dlext}",
+		PRE_WHOLE_ARCHIVE	=>  '-Wl,--whole-archive',
+		POST_WHOLE_ARCHIVE	=>  '-Wl,--no-whole-archive',
+		PERL_PARSE_DUP_ENV	=>  '-DPERL_PARSE_DUP_ENV',
 	} ;
 
 	my $map = {
@@ -175,16 +177,23 @@ sub portable {
 			GOT_NEXT_FREE_PORT	=>	0,
 			GOT_SYMLINK			=>	0,
 			GOT_SAFE_SIGNALS	=>	0,
-			SUB_FIX_CMD_QUOTES	=>	($COMMAND_COM ? undef : sub {
-				my $val = shift ;
-				$val = qq{"$val"} ;
-				return $val ;
-			}),
+
+# Can't remember what this was supposed to fix, but it breaks
+# when there are spaces in the J2SDK directory...
+#
+#			SUB_FIX_CMD_QUOTES	=>	($COMMAND_COM ? undef : sub {
+#				my $val = shift ;
+#				$val = qq{"$val"} ;
+#				return $val ;
+#			}),
+#
 			SUB_FIX_MAKE_QUOTES	=>	sub {
 				my $val = shift ;
 				$val = qq{"$val"} ;
 				return $val ;
 			},
+			PRE_WHOLE_ARCHIVE	=>  '',
+			POST_WHOLE_ARCHIVE	=>  '',
 		},
 		cygwin => {
 			ENV_VAR_PATH_SEP_CP	=>	';',
@@ -204,6 +213,8 @@ sub portable {
 		},
 		solaris => {
 			GOT_NEXT_FREE_PORT  =>  0,
+			PRE_WHOLE_ARCHIVE	=>  '-Wl,-zallextract',
+			POST_WHOLE_ARCHIVE	=>  '-Wl,-zdefaultextract',
 		},
 		aix => {
 			JVM_LIB				=>	"libjvm$Config{lib_ext}",
