@@ -5,7 +5,7 @@ use Inline::Java::Object ;
 use Inline::Java::Array ;
 use Carp ;
 
-$Inline::Java::Protocol::VERSION = '0.47' ;
+$Inline::Java::Protocol::VERSION = '0.49' ;
 
 my %CLASSPATH_ENTRIES = () ;
 
@@ -79,6 +79,15 @@ sub ISA {
 
 	my $class = $this->{obj_priv}->{java_class} ;
 
+	return $this->__ISA($proto, $class) ;
+}
+
+
+sub __ISA {
+	my $this = shift ;
+	my $proto = shift ;
+	my $class = shift ;
+
 	Inline::Java::debug(3, "checking if $class is a $proto") ;
 
 	my $data = join(" ", 
@@ -148,6 +157,24 @@ sub CallJavaMethod {
 }
 
 
+# Casts a Java object.
+sub Cast {
+	my $this = shift ;
+	my $class = shift ;
+
+	my $id = $this->{obj_priv}->{id} ;
+	Inline::Java::debug(3, "creating a new reference to object($id) with type $class") ;
+
+	my $data = join(" ", 
+		"cast", 
+		$id,
+		Inline::Java::Class::ValidateClass($class),
+	) ;
+
+	return $this->Send($data) ;
+}
+
+
 # Sets a member variable.
 sub SetJavaMember {
 	my $this = shift ;
@@ -255,22 +282,30 @@ sub ValidateArgs {
 			push @ret, "undef:" ;
 		}
 		elsif (ref($arg)){
-			if ((! UNIVERSAL::isa($arg, "Inline::Java::Object"))&&(! UNIVERSAL::isa($arg, "Inline::Java::Array"))){
+			if ((UNIVERSAL::isa($arg, "Inline::Java::Object"))||(UNIVERSAL::isa($arg, "Inline::Java::Array"))){
+				my $obj = $arg ;
+				if (UNIVERSAL::isa($arg, "Inline::Java::Array")){
+					$obj = $arg->__get_object() ; 
+				}
+				my $class = $obj->__get_private()->{java_class} ;
+				my $id = $obj->__get_private()->{id} ;
+				push @ret, "java_object:$class:$id" ;
+			}
+			elsif ($arg =~ /^(.*?)=/){
+				my $id = Inline::Java::Callback::PutObject($arg) ;
+				# Bug. The delimiter is :, so we need to escape the package separator (::)
+				my $pkg = $1 ;
+				$pkg =~ s/:/\//g ;
+				push @ret, "perl_object:$pkg:$id" ;
+			}
+			else {
 				if (! $callback){
-					croak "A Java method or member can only have Java objects, Java arrays or scalars as arguments" ;
+					croak "A Java method or member can only have Java objects, Java arrays, Perl objects or scalars as arguments" ;
 				}
 				else{
-					croak "A Java callback function can only return Java objects, Java arrays or scalars" ;
+					croak "A Java callback function can only return Java objects, Java arrays, Perl objects or scalars" ;
 				}
 			}
-
-			my $obj = $arg ;
-			if (UNIVERSAL::isa($arg, "Inline::Java::Array")){
-				$obj = $arg->__get_object() ; 
-			}
-			my $class = $obj->__get_private()->{java_class} ;
-			my $id = $obj->__get_private()->{id} ;
-			push @ret, "object:$class:$id" ;
 		}
 		else{
 			push @ret, "scalar:" . encode($arg) ;
@@ -324,7 +359,7 @@ sub DeserializeObject {
 	elsif ($resp =~ /^undef:$/){
 		return undef ;
 	}
-	elsif ($resp =~ /^object:([01]):(\d+):(.*)$/){
+	elsif ($resp =~ /^java_object:([01]):(\d+):(.*)$/){
 		# Create the Perl object wrapper and return it.
 		my $thrown = $1 ;
 		my $id = $2 ;
@@ -362,6 +397,7 @@ sub DeserializeObject {
 				if (Inline::Java::Class::ClassIsReference($elem_class)){
 					if (! Inline::Java::known_to_perl($pkg, $elem_class)){
 						if (($thrown)||($this->{inline}->get_java_config('AUTOSTUDY'))){
+							Inline::Java::debug(2, "autostudying $elem_class...") ;
 							$this->{inline}->_study([$elem_class]) ;
 						}
 						else{	
@@ -404,6 +440,12 @@ sub DeserializeObject {
 			}
 		}
 	}
+	elsif ($resp =~ /^perl_object:(\d+):(.*)$/){
+		my $id = $1 ;
+		my $pkg = $2 ;
+
+		return Inline::Java::Callback::GetObject($id) ;
+	}
 	else{
 		croak "Malformed response from server: $resp" ;
 	}
